/***************************************************************************//**
 *   @file   Main.c
 *   @brief  Implementation of the program's main function.
 *   @author ACozma (andrei.cozma@analog.com)
********************************************************************************
 * Copyright 2012(c) Analog Devices, Inc.
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *  - Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  - Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  - Neither the name of Analog Devices, Inc. nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *  - The use of this software may or may not infringe the patent rights
 *    of one or more patent holders.  This license does not release you
 *    from the requirement that you obtain separate licenses from these
 *    patent holders to use this software.
 *  - Use of the software either in source or binary form, must be run
 *    on or directly connected to an Analog Devices Inc. component.
 *
 * THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT,
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
********************************************************************************
 *   SVN Revision: 570
*******************************************************************************/

/******************************************************************************/
/* Include Files                                                              */
/******************************************************************************/
#include "YRDKRX62N.h"
#include "ADAS1000.h"

/******************************************************************************/
/* Variables Declarations                                                     */
/******************************************************************************/
unsigned char	dataBuffer[24000] = {0, };
unsigned char	ascii[3] = {0, 0, 0};
unsigned short	frame = 0;
unsigned char	character = 0;
unsigned long	crcValue = 0;

/***************************************************************************//**
 * @brief Converts desired decimal number into ascii code.
 *
 * @param decimal - The decimal number.
 * @param ascii - Ascii buffer.
 *
 * @return None.
*******************************************************************************/
void decimalToAscii(unsigned char decimal, unsigned char* ascii)
{
	ascii[0] = (decimal / 100) % 10 + 0x30;
	ascii[1] = (decimal / 10) % 10 + 0x30;
	ascii[2] = (decimal / 1) % 10 + 0x30;
}

/***************************************************************************//**
 * @brief Main function.
 *
 * @return None.
*******************************************************************************/
void main(void)
{
/* Local variables*/
	unsigned long regVal = 0;	
	
/* RX62N board initialization */	
	YRDKRX62N_Init();
	SCI_Init();

/* Initialize the communication with the ADAS1000, set the frame rate to 2KHz */
	ADAS1000_Init(ADAS1000_2KHZ_FRAME_RATE);

/* Example on how to read and write a register */
	// Write the CMREFCTL register with the value 0x85E0000B
	ADAS1000_SetRegisterValue(ADAS1000_CMREFCTL, 0x85E0000B);	
	//Read back the value of the CMREFCTL register
	ADAS1000_GetRegisterValue(ADAS1000_CMREFCTL, &regVal);	
	//Write a NOP command to flush out the data on SDO
	ADAS1000_SetRegisterValue(ADAS1000_NOP, 0);
		
/* Example how to start the ADCs Converting and begin streaming ECG data from ADAS1000 at 2kHz rate */
	//Power up device, enable all ECG channels in gain 1.4, low noise mode, 
	//Master device using XTAL input source
	ADAS1000_SetRegisterValue(ADAS1000_ECGCTL, 0x81F800AE);
	//Read a frame of data from the ADAS1000
	ADAS1000_ReadData(dataBuffer, 1,
					  1, 1,
					  0, 0);
	//Compute the CRC of the frame
	crcValue = ADAS1000_ComputeFrameCrc(dataBuffer);

/* Example on how to Enable Respiration, read a data frame */
	//Configures 56kHz respiration signal with Gain 1, 
	//internal respiration cap selected, LEAD I measure path
	ADAS1000_SetRegisterValue(ADAS1000_RESPCTL, 0x8302099);
	//Read a frame of data from the ADAS1000
	ADAS1000_ReadData(dataBuffer, 1,
					  1, 1,
					  0, 0);
	//Compute the CRC of the frame
	crcValue = ADAS1000_ComputeFrameCrc(dataBuffer);

/* Example on how to enable DC Leads Off, read a data frame */
	//DC Leads off enabled, leads off current 50nA
	ADAS1000_SetRegisterValue(ADAS1000_LOFFCTL, 0x82000015);
	//Read a frame of data from the ADAS1000
	ADAS1000_ReadData(dataBuffer, 1,
					  1, 1,
					  0, 0);
	//Compute the CRC of the frame
	crcValue = ADAS1000_ComputeFrameCrc(dataBuffer);

/* Example on how to enable Shield Amplifier, read a data frame */
	//DC Leads off enabled, leads off current 50nA
	ADAS1000_SetRegisterValue(ADAS1000_CMREFCTL, 0x85E0000B);
	//Read a frame of data from the ADAS1000
	ADAS1000_ReadData(dataBuffer, 1,
					  1, 1,
					  0, 0);
	//Compute the CRC of the frame
	crcValue = ADAS1000_ComputeFrameCrc(dataBuffer);

/* Example on how to change the number of words in a frame, read a data frame */
	//Disable the Respiration Magnitude and Respiration Phase words
	ADAS1000_SetInactiveFrameWords(ADAS1000_FRMCTL_RESPMDIS | 
								   ADAS1000_FRMCTL_RESPPHDIS); 
	//Read a frame of data from the ADAS1000
	ADAS1000_ReadData(dataBuffer, 1,
					  1, 1,
					  0, 0);
	//Compute the CRC of the frame
	crcValue = ADAS1000_ComputeFrameCrc(dataBuffer);

/* From power up, configure device and start streaming ECG data at 2kHz rate 
   with Leads off enabled, respiration enabled (all of above for the following configuration) */										
	//Initialize the communication with the ADAS1000, set the frame rate to 2KHz
	ADAS1000_Init(ADAS1000_2KHZ_FRAME_RATE);
	//Configure VCM = WCT = (LA+LL+RA)/3. RLD is enabled onto RL, Shield amplifier enabled. 
	ADAS1000_SetRegisterValue(ADAS1000_CMREFCTL, 0x85E0000B);
	//DC Leads off enabled, leads off current 50nA
	ADAS1000_SetRegisterValue(ADAS1000_LOFFCTL, 0x82000015);
	//Configures 56kHz respiration signal with Gain 1, 
	//internal respiration cap selected, LEAD I measure path
	ADAS1000_SetRegisterValue(ADAS1000_RESPCTL, 0x002099);
	//Powers up device, enables all ECG channels in gain 1.4, 
	//low noise mode, Master device using XTAL input source
	ADAS1000_SetRegisterValue(ADAS1000_ECGCTL, 0x81F800AE);
	//Read a frame of data from the ADAS1000
	ADAS1000_ReadData(dataBuffer, 1,
					  1, 1,
					  0, 0);
	//Compute the CRC of the frame
	crcValue = ADAS1000_ComputeFrameCrc(dataBuffer);

/* Configure device and start streaming ECG data at 16kHz rate */
	//Change the frame rate to 16KHz
	ADAS1000_SetFrameRate(ADAS1000_16KHZ_FRAME_RATE);
	//Read a frame of data from the ADAS1000
	ADAS1000_ReadData(dataBuffer, 1,
					  1, 1,
					  0, 0);
	//Compute the CRC of the frame
	crcValue = ADAS1000_ComputeFrameCrc(dataBuffer);

/* Configure device and start streaming ECG data at 128kHz rate */
	//Change the frame rate to 16KHz
	ADAS1000_SetFrameRate(ADAS1000_128KHZ_FRAME_RATE);
	//Read a frame of data from the ADAS1000
	ADAS1000_ReadData(dataBuffer, 1,
					  1, 1,
					  0, 0);
	//Compute the CRC of the frame
	crcValue = ADAS1000_ComputeFrameCrc(dataBuffer);

/* Configure 150Hz test tone Sinewave on each ECG channel and stream data */				  
	//Initialize the communication with the ADAS1000, set the frame rate to 16KHz
	ADAS1000_Init(ADAS1000_16KHZ_FRAME_RATE);	
	//Reduce the frame size to the minimum
	ADAS1000_SetInactiveFrameWords(ADAS1000_FRMCTL_PACEDIS | ADAS1000_FRMCTL_RESPMDIS | 
								   ADAS1000_FRMCTL_RESPPHDIS |	ADAS1000_FRMCTL_LOFFDIS | 
								   ADAS1000_FRMCTL_GPIODIS | ADAS1000_FRMCTL_CRCDIS);	
	//Enable the electrode mode in the FRMCTL register
	ADAS1000_GetRegisterValue(ADAS1000_FRMCTL, &regVal);
	regVal |= ADAS1000_FRMCTL_DATAFMT;
	ADAS1000_SetRegisterValue(ADAS1000_FRMCTL, regVal);	
	//Configure VCM = WCT = (LA+LL+RA)/3. RLD is enabled onto RL, Shield amplifier enabled. 
	ADAS1000_SetRegisterValue(ADAS1000_CMREFCTL, 0x8500000B);
	//150Hz sinewave put onto each electrode. 
	ADAS1000_SetRegisterValue(ADAS1000_TESTTONE, 0x88F8000D);
	//Change internal LPF to 250Hz
	ADAS1000_SetRegisterValue(ADAS1000_FILTCTL, 0x8B000008);	
	//Powers up device, enables all ECG channels in gain 1.4, 
	//low noise mode, Master device using XTAL input source
	ADAS1000_SetRegisterValue(ADAS1000_ECGCTL,   0x81F800AE);

	//Start the data streaming
	ADAS1000_ReadData(dataBuffer, 1000,
					  1, 1,
					  1, 0);
	//Ouput the on the UART the frames read from the ADAS1000
	for(frame = 0; frame < 24000; frame++)
	{
		decimalToAscii(dataBuffer[frame], ascii);
		SCI_Write((unsigned char*)&ascii[0], 1);
		SCI_Write((unsigned char*)&ascii[1], 1);
		SCI_Write((unsigned char*)&ascii[2], 1);
		character = 0x2C;	// , (comma)
		SCI_Write((unsigned char*)&character, 1);
		character = 0x20;	// SP  (Space)
		SCI_Write((unsigned char*)&character, 1);
		if ((frame + 1) % 24 == 0)
		{
			character = 0x0D; // CR (Carriage Return).
			SCI_Write((unsigned char*)&character, 1);
			character = 0x0A; // LF (Line Feed).
			SCI_Write((unsigned char*)&character, 1);
		}
	}
	//Wait indefinitely
	while(1)
	{
	}
}
